
require 'dotnet'

reference 'System.Windows.Forms'

module Ridge
	module GUI
		def message(str)
			System::Windows::Forms::MessageBox.show(str)
		end
		module_function :message
		
		module DotNetControl
			private
			def set_text(properties)
				ctrl = self.native
				ctrl.text = properties['text'] if properties['text']
			end

			def set_positon(properties)
				ctrl = self.native
				ctrl.left = Length.to_px(properties['left']) if properties['left']
				ctrl.top = Length.to_px(properties['top']) if properties['top']
				ctrl.width = Length.to_px(properties['width']) if properties['width']
				ctrl.height = Length.to_px(properties['height']) if properties['height']
				ctrl.right = Length.to_px(properties['right']) if properties['right']
				ctrl.bottom = Length.to_px(properties['bottom']) if properties['bottom']
				if properties['anchors'] then
					anchors = properties['anchors'].split(' ')
					anchor = []
					anchors.each do |a|
						case a.strip.downcase
						when 'left'
							anchor.push('Left')
						when 'top'
							anchor.push('Top')
						when 'right'
							anchor.push('Right')
						when 'bottom'
							anchor.push('Bottom')
						end
					end
					ctrl.anchor = 
						System::Enum.parse(System::Windows::Forms::AnchorStyles, anchor.join(','))
				end
				if properties['dock'] then
					dock = nil 
					case properties['dock'].downcase
					when 'left'
						dock = System::Windows::Forms::DockStyle.Left
					when 'top'
						dock = System::Windows::Forms::DockStyle.Top
					when 'right'
						dock = System::Windows::Forms::DockStyle.Right
					when 'bottom'
						dock = System::Windows::Forms::DockStyle.Bottom
					when 'fill'
						dock = System::Windows::Forms::DockStyle.Fill
					end
					ctrl.dock = dock if dock
				end
			end

			def set_children(children, reverse=false)
				if not reverse then
					children.each() do |child|
							child.attach_to(self)
					end
				else
					children.reverse.each do |child|
						child.attach_to(self)
					end
					children.each_with_index do |child, i|
						begin
							child.native.tabindex = i
						rescue DotNet::Exception
							#do nothing for which not has tabindex property.
						end
					end
				end
			end

			def default_handler(name)
				proc {|*args| @resource.events.fire(@properties[name], *args)}
			end
		end
		
		class Window < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::Form.new()
			end
			
			def set_event()
				@window.add_Activated proc {|sender, args| @events.fire('on_active', self, {}) }
				@events.add_handler('on_active', default_handler('on_active'))
			end
			
			def initialize(rc, properties, children=[])
				super
				@window = create_widget()
				@window.tag = self
				if properties['icon'] then
					icon = rc.image(properties['icon'])
					@window.icon = icon.new.native if icon
				end
				@window.startPosition = System::Windows::Forms::FormStartPosition.WindowsDefaultLocation
				set_text(properties)
				set_positon(properties)
				sz = @window.clientSize
				sz.width = Length.to_px(properties['width']) if properties['width']
				sz.height = Length.to_px(properties['height']) if properties['height']
				if properties['fixed'] then
					@fixed = Boolean.parse(properties['fixed'])
					if @fixed then
						@window.formBorderStyle = System::Windows::Forms::FormBorderStyle.FixedDialog
						@window.maximizeBox = false
						@window.minimizeBox = false
					end
				else
					@fixed = false
				end
				@window.clientSize = sz
				set_children(@children, true)
				set_event()
			end
			attr :children

			def native()
				@window
			end
			
			def attach_to(parent)
				parent.native.addOwnedForm(@window)
				if @fixed then
					@window.formBorderStyle = System::Windows::Forms::FormBorderStyle.FixedToolWindow
				else
					@window.formBorderStyle = System::Windows::Forms::FormBorderStyle.SizableToolWindow
				end
				@window.show
			end
			
			def show()
				@window.show
			end

			def show_modal()
				@window.showDialog
			end
			
			def close()
				@window.close
			end
			
			def run()
				#Thread΍
				#Application.run(@window)ɂƁA
				#RubỹXbh肭ȂȂB
				self.show
				while @window.visible
					System::Windows::Forms::Application.doEvents()
					sleep(0.01)
				end
			end
		end
		Widgets['window'] = Window

		class Dialog < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::Form.new()
			end
			
			def set_event()
			end
			
			def initialize(rc, properties, children=[])
				super
				@window = create_widget()
				@window.tag = self
				if properties['icon'] then
					icon = rc.image(properties['icon'])
					@window.icon = icon.new.native if icon
				end
				@window.startPosition = System::Windows::Forms::FormStartPosition.WindowsDefaultLocation
				set_text(properties)
				set_positon(properties)
				sz = @window.clientSize
				sz.width = Length.to_px(properties['width']) if properties['width']
				sz.height = Length.to_px(properties['height']) if properties['height']
				@window.formBorderStyle = System::Windows::Forms::FormBorderStyle.FixedDialog
				@window.maximizeBox = false
				@window.minimizeBox = false
				@window.clientSize = sz
				set_children(@children, true)
				set_event()
			end
			attr :children

			def native()
				@window
			end
			
			def attach_to(parent)
				parent.native.addOwnedForm(@window)
				@window.show
			end
			
			def show()
				@window.showDialog
			end

			def close()
				@window.close
			end
		end
		Widgets['dialog'] = Dialog

		class MDIWindow < Window
			def create_widget()
				widget = super
				widget.isMdiContainer = true
				return widget
			end

			def attach_to(parent)
				@window.isMdiContainer = false
				@window.mdiParent = parent.native
				@window.show
			end
		end
		Widgets['mdiwindow'] = MDIWindow

		class Button < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::Button.new()
			end

			def set_event()
				@button.add_Click proc {|sender, args| @events.fire('on_click', self, {})}
				@events.add_handler('on_click', default_handler('on_click'))
			end
			
			def initialize(rc, properties, children=[])
				super
				@button = create_widget()
				@button.tag = self
				set_positon(properties)
				set_text(properties)
				set_event()
			end

			def attach_to(parent)
				parent.native.controls.add(@button)
			end
			
			def native()
				@button
			end
		end
		Widgets['button'] = Button

		class Text < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::TextBox.new()
			end

			def set_event()
				@text.add_TextChanged proc {|sender, args| @events.fire('on_change', self, 'value'=>self.text)}
				@events.add_handler('on_change', default_handler('on_change'))
			end
			
			def initialize(rc, properties, children=[])
				super
				@text = create_widget()
				@text.tag = self
				set_positon(properties)
				set_text(properties)
				set_event()
			end

			def native()
				@text
			end

			def attach_to(parent)
				parent.native.controls.add(@text)
			end
			
			def text()
				@text.text
			end

			def text=(val)
				@text.text = val.to_s
			end
		end
		Widgets['text'] = Text

		class MultiText < Text
			def create_widget()
				widget = super
				widget.multiline = true
				return widget
			end
		end
		Widgets['multitext'] = MultiText

		class Label < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::Label.new()
			end

			def set_event()
			end

			def initialize(rc, properties, children=[])
				super
				@label = create_widget()
				@label.tag = self
				@label.autosize = true
				@label.backColor = System::Drawing::Color.transparent
				set_positon(properties)
				set_text(properties)
				set_event()
			end

			def native()
				@label
			end

			def attach_to(parent)
				parent.native.controls.add(@label)
			end
			
			def text()
				@label.text
			end

			def text=(val)
				@label.text = val.to_s
			end
		end
		Widgets['label'] = Label

		class ImageBox < Widget
			include DotNetControl
			def create_widget()
				System::Windows::Forms::PictureBox.new()
			end

			def set_event()
			end

			def initialize(rc, properties, children=[])
				@picturebox = create_widget()
				@picturebox.tag = self
				@image = nil
				@picturebox.sizeMode = System::Windows::Forms::PictureBoxSizeMode.AutoSize
				if properties['image'] then
					img = rc.image(properties['image'])
					if img then
						@image = img.new
						@picturebox.image = @image.native
					end
				end
				if properties['width'] || properties['height'] || 
					 properties['right'] || properties['bottom'] ||
					 properties['anchor'] || properties['dock'] then
					@picturebox.sizeMode = System::Windows::Forms::PictureBoxSizeMode.StretchImage
				end
				set_positon(properties)
				set_event()
			end

			def native()
				@picturebox
			end

			def image()
				@image
			end

			def image=(val)
				@picturebox.image = val.native
				@image = val
			end

			def attach_to(parent)
				parent.native.controls.add(@picturebox)
			end
		end
		Widgets['imagebox'] = ImageBox

		class CheckBox < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::CheckBox.new()
			end

			def set_event()
				@check.add_CheckedChanged proc {|sender, args| @events.fire('on_change', self, 'value'=>self.checked)}
				@events.add_handler('on_change', default_handler('on_change'))
			end

			def initialize(rc, properties, children=[])
				super
				@check = create_widget()
				@check.tag = self
				set_positon(properties)
				set_text(properties)
				@check.checked = Boolean.parse(properties['checked']) ? true : false
				set_event()
			end

			def attach_to(parent)
				parent.native.controls.add(@check)
			end
			
			def checked()
				@check.checked
			end
			
			def checked=(val)
				@check.checked = val ? true : false
			end

			def native()
				@check
			end
		end
		Widgets['checkbox'] = CheckBox

		class RadioBox < CheckBox
			def create_widget()
				System::Windows::Forms::RadioButton.new()
			end

			def initialize(rc, properties, children=[])
				super
				@value = properties['value'] if properties['value']
			end
			attr :value, true
		end
		Widgets['radiobox'] = RadioBox

		class OptionGroup < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::Panel.new()
			end

			def set_event()
				@children.each do |child|
					child.events.add_handler('on_change', proc { 
						@resource.events.fire(@properties['on_change'], self, 'value'=>self.selected)
					})
				end
			end
			
			def initialize(rc, properties, children=[])
				super
				@panel = create_widget()
				@panel.tag = self
				set_positon(properties)
				set_children(@children, true)
				set_event()
			end
			attr :children

			def native()
				return @panel
			end

			def selected()
				return @children.select() {|child| child.checked }
			end
			
			def attach_to(parent)
				parent.native.controls.add(@panel)
			end
		end
		Widgets['optiongroup'] = OptionGroup

		class List < Widget
			include DotNetControl

			def create_widget()
				list = System::Windows::Forms::ListBox.new()
				list.integralHeight = false
				return list
			end
			
			def set_event()
				@list.add_SelectedIndexChanged proc {|sender, args| @events.fire('on_change', self, 'value'=>self.selected)}
				@events.add_handler('on_change', default_handler('on_change'))
			end

			def initialize(rc, properties, children=[])
				super
				@list = create_widget()
				@list.tag = self
				set_positon(properties)
				set_children(@children)
				self.selected = @children[properties['selected'].to_i] if properties['selected']
				set_event()
			end
			attr :children

			def native()
				@list
			end

			def selected()
				if @list.selectedIndex<0 then
					return nil
				else
					return @children[@list.selectedIndex]
				end
			end

			def selected=(item)
				index = @children.index(item)
				if index then
					@list.selectedIndex = index
				end
			end

			def add(item)
				@children.push(item)
				item.attach_to(self)
			end

			def remove(item)
				index = @children.index(item)
				if index then
					@children.delete_at(index)
					@list.items.removeAt(index)
				end
			end

			def insert(positm, item)
				index = @children.index(positm)
				if index then
					@children.insert(index, item)
					item.attach_to(self, index)
				else
					self.add(item)
				end
			end
			
			def attach_to(parent)
				parent.native.controls.add(@list)
			end
		end
		Widgets['list'] = List

		class DropDownList < List
			def create_widget()
				list = System::Windows::Forms::ComboBox.new()
				list.dropDownStyle = System::Windows::Forms::ComboBoxStyle.DropDownList
				return list
			end
		end
		Widgets['dropdownlist'] = DropDownList
		
		class ComboBox < List
			def create_widget()
				list = System::Windows::Forms::ComboBox.new()
				list.dropDownStyle = System::Windows::Forms::ComboBoxStyle.DropDown
				return list
			end

			def set_event()
				@list.add_TextChanged proc {|sender, args| @events.fire('on_change', self, 'value'=>self.text)}
				@events.add_handler('on_change', default_handler('on_change'))
			end

			def initialize(rc, properties, children=[])
				super
				@list.tag = self
				set_text(properties)
			end

			def text()
				@list.text
			end

			def text=(val)
				@list.text = val.to_s
			end
		end
		Widgets['combobox'] = ComboBox
		
		class ListItem < Widget
			def initialize(rc, properties, children=[])
				super
				@text = properties['text']
				if properties['value'] then
					@value = properties['value']
				else
					@value = @text
				end
			end
			attr :value, true
			attr :text

			def attach_to(parent, pos=nil)
				if pos then
					parent.native.items.insert(pos, @text)
				else
					parent.native.items.add(@text)
				end
			end
		end
		Widgets['listitem'] = ListItem

		class MenuItem < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::MenuItem.new()
			end

			def set_event()
				@menuitem.add_Click proc {|sender, args| @events.fire('on_click', self, {})}
				@events.add_handler('on_click', default_handler('on_click'))
			end

			def initialize(rc, properties, children=[])
				super
				@menuitem = create_widget()
				set_text(properties)
				set_children(@children)
				set_event()
			end
			attr :children

			def native()
				return @menuitem
			end

			def attach_to(parent)
				parent.native.menuItems.add(@menuitem)
			end
		end
		Widgets['menuitem'] = MenuItem
		
		class MainMenu < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::MainMenu.new()
			end

			def set_event()
			end

			def initialize(rc, properties, children=[])
				super
				@menu = create_widget()
				set_children(children)
				set_event()
			end
			attr :children

			def native()
				return @menu
			end

			def attach_to(parent)
				parent.native.menu = @menu
			end
		end
		Widgets['mainmenu'] = MainMenu

		class Panel < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::Panel.new()
			end

			def set_event()
			end

			def initialize(rc, properties, children=[])
				super
				@panel = create_widget()
				@panel.tag = self
				set_positon(properties)
				set_children(@children, true)
				set_event()
			end
			attr :children

			def native()
				return @panel
			end

			def attach_to(parent)
				parent.native.controls.add(@panel)
			end
		end
		Widgets['panel'] = Panel
		
		class ToolBar < Widget
			include DotNetControl

			def create_widget()
				toolbar = System::Windows::Forms::ToolBar.new()
				toolbar.appearance = System::Windows::Forms::ToolBarAppearance.Flat
				toolbar.imageList = System::Windows::Forms::ImageList.new()
				return toolbar
			end

			def set_event()
				@toolbar.add_ButtonClick proc {|sender, args| 
					args.button.tag.events.fire('on_click', args.button.tag, {})
				}
			end

			def initialize(rc, properties, children=[])
				super
				@toolbar = create_widget()
				@toolbar.tag = self
				set_positon(properties)
				set_children(@children)
				set_event()
			end
			attr :children

			def native()
				return @toolbar
			end

			def attach_to(parent)
				parent.native.controls.add(@toolbar)
			end
		end
		Widgets['toolbar'] = ToolBar
		
		class ToolButton < Widget
			include DotNetControl
			def create_widget()
				System::Windows::Forms::ToolBarButton.new()
			end

			def set_event()
				@events.add_handler('on_click', default_handler('on_click'))
			end

			def initialize(rc, properties, children=[])
				super
				@button = create_widget()
				@button.tag = self
				if properties['text'] then
					if properties['text']=='-' then
						@button.style = System::Windows::Forms::ToolBarButtonStyle.Separator
					else
						@button.toolTipText = properties['text']
					end
				end
				@image = nil
				if properties['image'] then
					imgrc = rc.image(properties['image'])
					@image = imgrc.new if imgrc
				else
					@button.text = properties['text']
				end
				set_event()
			end

			def native()
				return @button
			end

			def attach_to(parent)
				if @image then
					parent.native.imageList.images.add(@image.native)
					idx = parent.native.imageList.images.count - 1
					@button.imageIndex = idx
				end
				parent.native.buttons.add(@button)
			end
		end
		Widgets['toolbutton'] = ToolButton

		class ToolCheckButton < ToolButton
			include DotNetControl
			def create_widget()
				button = System::Windows::Forms::ToolBarButton.new()
				button.style = System::Windows::Forms::ToolBarButtonStyle.ToggleButton
			end

			def initialize(rc, properties, children=[])
				super
				if properties['checked'] then
					@button.pushed = Boolean.parse(properties['checked'])
				end
			end

			def checked()
				@button.pushed
			end

			def checked=(val)
				@button.pushed = val ? true : false
			end
		end
		Widgets['toolcheckbutton'] = ToolCheckButton
		
		class Pane < Widget
			include DotNetControl

			def create_widget()
				System::Windows::Forms::Panel.new()
			end

			def set_event()
			end

			def initialize(rc, properties, children=[])
				super
				@panel = create_widget()
				@panel.tag = self
				@splitter = System::Windows::Forms::Splitter.new()
				set_positon(properties)
				@panel.dock = System::Windows::Forms::DockStyle.Left if properties['dock'].nil?
				@splitter.dock = @panel.dock
				set_children(@children, true)
				set_event()
			end
			attr :children

			def native()
				return @panel
			end

			def attach_to(parent)
				parent.native.controls.add(@splitter)
				parent.native.controls.add(@panel)
			end
		end
		Widgets['pane'] = Pane

		class SplitterPanel < Widget
			include DotNetControl
			def create_widget()
				System::Windows::Forms::Panel.new()
			end

			def set_event()
			end

			def initialize(rc, properties, children=[])
				super
				@mainpanel = create_widget()
				@mainpanel.tag = self
				@splitter = System::Windows::Forms::Splitter.new()
				set_positon(properties)
				@subpanel = []
				sub_prop = [{}, {}]
				sub_child = [[], []]
				if Boolean.parse(properties['vertical']) then
					sub_prop[0]['dock'] = 'top'
					sub_prop[0]['height'] = properties['split']
					sub_prop[1]['dock'] = 'fill'
					@splitter.dock = System::Windows::Forms::DockStyle.Top
				else
					sub_prop[0]['dock'] = 'left'
					sub_prop[0]['width'] = properties['split']
					sub_prop[1]['dock'] = 'fill'
					@splitter.dock = System::Windows::Forms::DockStyle.Left
				end
				@children = []
				warn 'Too many children for splitterpanel' if children.size > 2
				sz = children.size>2 ? 2 : children.size
				sz.times do |i|
					child = children[i]
					@children.push(child)
					sub_child[i].push(child)
				end
				@subpanel[0] = Widgets['panel'].new(rc, sub_prop[0], sub_child[0])
				@subpanel[1] = Widgets['panel'].new(rc, sub_prop[1], sub_child[1])
				@mainpanel.controls.add(@subpanel[1].native)
				@mainpanel.controls.add(@splitter)
				@mainpanel.controls.add(@subpanel[0].native)
				set_event()
			end
			attr :children

			def native()
				return @mainpanel
			end

			def attach_to(parent)
				parent.native.controls.add(@mainpanel)
			end
		end
		Widgets['splitterpanel'] = SplitterPanel
	end
end

